# -*- coding: utf-8 -*-

from l10n_ar_api.documents import tribute
import pytest


class TestProfit:

    @pytest.fixture(scope="module")
    def profit(self):
        return tribute.Tribute.get_tribute('profit')

    @pytest.fixture(scope="module")
    def goods(self):
        return tribute.Activity(100000, 90, 2.0)

    def test_calculate_retention_without_activity(self, profit):
        with pytest.raises(AttributeError):
            profit.calculate_value(0, 0)

    """ Tests sin acumulados """
    def test_calculate_retention_without_accumulated_less_than_minimum(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(0, 50000)[1] == 0

    def test_calculate_retention_without_accumulated_less_than_minimum_tax(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(0, 102000)[1] == 0

    def test_calculate_retention_without_accumulated(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(0, 200000)[1] == 2000

    """ Tests con acumulados """
    def test_calculate_retention_with_accumulated_less_than_minimum(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(40000, 50000)[1] == 0

    def test_calculate_retention_with_accumulated_less_than_minimum_tax(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(50000, 70000)[1] == 400

    def test_calculate_retention_with_accumulated_less_than_minimum_tax_no_val(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(50000, 52000)[1] == 0

    def test_calculate_retention_with_accumulated_equal_than_minimum(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(100000, 5000)[1] == 100

    def test_calculate_retention_with_accumulated_higher_than_minimum(self, profit, goods):
        profit.activity = goods
        assert profit.calculate_value(150000, 5000)[1] == 100

    def test_calculate_retention_with_accumulated_higher_than_minimum_without_retention_before(
            self, profit, goods):
        """
        En el caso que sea la primer retencion del mes, si ya sobrepaso el acumulado, se debe sumar lo no retenido
        """
        profit.activity = goods
        assert profit.calculate_value(102000, 5000)[1] == 140



from __future__ import absolute_import
from . import documents
from . import afip_webservices
from . import anmat_webservices
from . import arba_webservices
from . import presentations
from . import padron



# -*- coding: utf-8 -*-
# Segun manual para el desarrollador version 30/09/2014
from l10n_ar_api.afip_webservices import config
from zeep import Client
from .error import AfipError
from .invoice import ExportationElectronicInvoiceValidator


class WsfexInvoiceDetails(object):
    """
    Se encarga de asignar los detalles de una invoice en
    los factories de WSFEX.

    :param wsfex: Objeto Wsfex.
    :param invoice: Objeto ElectronicInvoice para tomar los detalles.
    :param pos: Numero de punto de venta.
    """

    def __init__(self, wsfex, invoice, pos):
        self.wsfex = wsfex
        self.pos = pos
        self.invoice = invoice
        self.detail = None

    def get_details(self):
        """ Devuelve los detalles completos para esa factura """

        self._set_details()
        return self.detail

    def _set_details(self):
        """ Completa los detalles a enviar segun los datos del documento """

        last_document_req = self.wsfex.get_last_number(self.pos, self.invoice.document_code)
        self.detail = self._get_document_request()
        self.detail.Id = self.wsfex.get_last_id().FEXResultGet.Id + 1
        self.detail.Cbte_Tipo = self.invoice.document_code
        self.detail.Fecha_cbte = self.invoice.document_date
        # Se cheque que el document_code sea 19 para saber que es una factura de exportacion y no sea ND expo ni NC expo
        if self.invoice.document_code == 19 and self.invoice.exportation_type in [2, 4]:
            self.detail.Fecha_pago = self.invoice.payment_due_date or self.invoice.document_date

        self.detail.Punto_vta = self.pos
        self.detail.Cbte_nro = last_document_req + 1
        self.detail.Tipo_expo = self.invoice.exportation_type
        self.detail.Permiso_existente = self.invoice.existent_permission
        self.detail.Dst_cmp = self.invoice.destiny_country
        self.detail.Cliente = self.invoice.customer_name
        self.detail.Cuit_pais_cliente = self.invoice.destiny_country_cuit
        self.detail.Domicilio_cliente = self.invoice.customer_street
        self.detail.Moneda_Id = self.invoice.mon_id
        self.detail.Moneda_ctz = round(self.invoice.mon_cotiz, 6)
        self.detail.Imp_total = self.invoice.get_total_amount()
        self.detail.Idioma_cbte = self.invoice.document_language
        self.detail.Incoterms = self.invoice.incoterms
        self._set_associated_documents()
        self._set_items()

    def _serialize_item(self, item):
        return self._get_item()(
            Pro_ds=item.description,
            Pro_qty=item.quantity,
            Pro_umed=item.measurement_unit,
            Pro_precio_uni=item.unit_price,
            Pro_total_item=round(item.total_price, 2),
            Pro_bonificacion=item.bonification
        )

    def _serialize_associated_document(self, associated_document):
        return self._get_associated_document()(
            Cbte_tipo=associated_document.document_type,
            Cbte_punto_vta=associated_document.point_of_sale,
            Cbte_nro=associated_document.number,
            Cbte_cuit=associated_document.cuit,
        )

    def _set_items(self):
        if self.invoice.array_items:
            self.detail.Items = self._get_item_array()([
                self._serialize_item(item) for item in self.invoice.array_items
            ])

    def _set_associated_documents(self):
        if self.invoice.associated_documents:
            self.detail.Cmps_asoc = self._get_associated_document_array()([
                self._serialize_associated_document(associated) for associated in self.invoice.associated_documents
            ])

    def _get_associated_document_array(self):
        return Client(self.wsfex.url).get_type('ns0:ArrayOfCmp_asoc')

    def _get_document_request(self):
        return Client(self.wsfex.url).type_factory('ns0').ClsFEXRequest()

    def _get_item_array(self):
        return Client(self.wsfex.url).get_type('ns0:ArrayOfItem')

    def _get_item(self):
        return Client(self.wsfex.url).get_type('ns0:Item')

    def _get_associated_document(self):
        return Client(self.wsfex.url).get_type('ns0:Cmp_asoc')


class Wsfex(object):
    """
    Factura electronica de exportacion.

    :param access_token: AccessToken - Token de acceso
    :param cuit: Cuit de la empresa
    :param homologation: Homologacion si es True
    :param url: Url de servicios para Wsfe
    """

    def __init__(self, access_token, cuit, homologation=True, url=None):

        if not url:
            self.url = config.service_urls.get('wsfexv1_homologation') if homologation \
                else config.service_urls.get('wsfexv1_production')
        else:
            self.url = url

        self.accessToken = access_token
        self.cuit = cuit
        self.auth_request = self._create_auth_request()

    def get_cae(self, invoices, pos):
        """
        :param invoices: Conjunto de Objetos ExportationElectronicInvoice, documentos a enviar a AFIP.
        :param pos: Numero de punto de venta.
        :returns str: Respuesta de AFIP sobre la validacion del documento.
        """

        # TODO: Ver como integramos el get_cae para ambos servicios
        self._validate_invoices(invoices)

        responses = []
        requests = []
        for invoice in invoices:
            request = self._create_cae_request(invoice, pos)
            response = Client(self.url).service.FEXAuthorize(self.auth_request, request)
            self._check_for_errors(response)
            responses.append(response)
            requests.append(request)

        return responses, requests

    def check_webservice_status(self):
        """
        Consulta el estado de los webservices de FEX de AFIP
        :raises Exception: Si alguno de los servidores esta caido
        """

        res = Client(self.url).service.FEXDummy()
        if res.AppServer != 'OK':
            raise Exception('El servidor de aplicaciones no se encuentra disponible. Intente mas tarde.')
        if res.DbServer != 'OK':
            raise Exception('El servidor de base de datos no se encuentra disponible. Intente mas tarde.')
        if res.AuthServer != 'OK':
            raise Exception('El servidor de auntenticacion no se encuentra disponible. Intente mas tarde.')

        return res

    def get_last_number(self, pos_number, document_type_number):
        """ Recupera el ultimo comprobante autorizado """

        last_document_req = self._set_last_document(pos_number, document_type_number)
        request = Client(self.url).service.FEXGetLast_CMP(last_document_req)

        self._check_for_errors(request)

        return request.FEXResult_LastCMP.Cbte_nro

    def get_last_id(self):
        """" Recupera el ultimo ID y su fecha """

        request = Client(self.url).service.FEXGetLast_ID(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_exportation_types(self):
        """ Recupera el listado de los tipos de exportacion y sus codigos """

        request = Client(self.url).service.FEXGetPARAM_Tipo_Expo(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_measurement_units(self):
        """ Devuelve las unidades de medida y su codigo """

        request = Client(self.url).service.FEXGetPARAM_UMed(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_point_of_sales(self):
        """ Recupera el listado de los puntos de venta registrados para fex """

        request = Client(self.url).service.FEXGetPARAM_PtoVenta(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_currencies(self):
        """ Recupera el listado de monedas y su codigo utilizable """

        request = Client(self.url).service.FEXGetPARAM_MON(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_incoterms(self):
        """ Recupera el listado Incoterms utilizables """

        request = Client(self.url).service.FEXGetPARAM_Incoterms(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_languages(self):
        """ Recupera el listado de los idiomas y sus codigos utilizables """

        request = Client(self.url).service.FEXGetPARAM_Idiomas(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_countries(self):
        """ Recupera el listado de paises """

        request = Client(self.url).service.FEXGetPARAM_DST_pais(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_countries_cuit(self):
        """ Recupera el listado CUIT de los paises """

        request = Client(self.url).service.FEXGetPARAM_DST_CUIT(self.auth_request)
        self._check_for_errors(request)

        return request

    def get_currency_value(self, currency):
        """ Recupera la cotizacion de la moneda consultada """

        request = Client(self.url).service.FEXGetPARAM_Ctz(self.auth_request, currency)
        self._check_for_errors(request)

        return request

    def get_document_codes(self):
        """ Recupera el listado de los tipos de comprobante y su codigo utilizables """

        request = Client(self.url).service.FEXGetPARAM_Cbte_Tipo(self.auth_request)
        self._check_for_errors(request)

        return request

    def _create_cae_request(self, invoice, pos):
        """
        :param invoice: ExportationElectronicInvoice.
        :param pos: Numero de punto de venta.
        :returns: FECAERequest / Envio de documentos para recibir el CAE.
        """

        return WsfexInvoiceDetails(self, invoice, pos).get_details()

    def _set_last_document(self, pos_number, document_type_number):
        ClsFEX_LastCMP = Client(self.url).get_type('ns0:ClsFEX_LastCMP')

        last_document_req = ClsFEX_LastCMP(
            Token=self.auth_request.Token,
            Sign=self.auth_request.Sign,
            Cuit=self.auth_request.Cuit,
            Pto_venta=pos_number,
            Cbte_Tipo=document_type_number
        )
        return last_document_req

    def show_error(self, response):
        if response.FEXErr.ErrMsg != 'OK':
            raise AfipError.parse_error(response)

    @staticmethod
    def _validate_invoices(invoices):
        invoiceValidator = ExportationElectronicInvoiceValidator()
        for invoice in invoices:
            invoiceValidator.validate_invoice(invoice)

    @staticmethod
    def _check_for_errors(request):
        if request.FEXErr.ErrMsg != 'OK':
            raise AfipError().parse_error(request)

    def _create_auth_request(self):
        """ Setea el ClsFEXAuthRequest, necesario para utilizar el resto de los metodos """

        ClsFEXAuthRequest = Client(self.url).get_type('ns0:ClsFEXAuthRequest')
        # ns0:ClsFEXAuthRequest(Token: xsd:string, Sign: xsd:string, Cuit: xsd:long)

        auth_request = ClsFEXAuthRequest(
            Token=self.accessToken.token,
            Sign=self.accessToken.sign,
            Cuit=self.cuit
        )
        return auth_request


class WsfexAssociatedDocument(object):

    def __init__(self, document_type, point_of_sale, number, cuit):
        self.document_type = document_type
        self.point_of_sale = point_of_sale
        self.number = number
        self.cuit = cuit



try:
    import urllib2 as urllib
except ImportError:
    import urllib.request as urllib

try:
    from BeautifulSoup import BeautifulSoup
except ImportError:
    from bs4 import BeautifulSoup

from unidecode import unidecode
from l10n_ar_api.padron.contributor import Contributor


class Banks:
             
    @staticmethod
    def get_banks_list():
        """
        Obtiene la lista de bancos desde AFIP utilizando
        la libreria BeautifulSoup para webscraping
        """

        url = 'http://www.afip.gov.ar/genericos/emisorasGarantias/formularioCompa%C3%B1ias.asp?completo=1&ent=3'
        req = urllib.Request(url)
        f = urllib.urlopen(req)
        soup = BeautifulSoup(f)
        table = soup.find('table', attrs={"class": "contenido"})
        banks = []
        for row in table.findAll('tr')[2:]:
            banks.append([td.text.strip() for td in row.findAll('td') if td.text.strip()])
                        
        return banks

    @staticmethod
    def get_values(banks_list):
        """
        :param banks_list: Lista de bancos.
        :return: Lista de diccionarios con los valores de cada banco
        """
        
        values = []
        for bank_list in banks_list:
            
            bank_values = {}
            
            if bank_list:    
                
                # Validamos el cuit del banco para comprobar existencia
                if Contributor.is_valid_cuit(bank_list[0]):
            
                    bank_values['cuit'] = bank_list[0]
                    bank_values['code'] = bank_list[1]
                    bank_values['name'] = unidecode(bank_list[2])
                    values.append(bank_values)
            
        return values



# -*- coding: utf-8 -*-
# Segun RG 2485 – Proyecto FE v2.8 - 12/09/2016

from zeep import Client

from .error import AfipError
from l10n_ar_api.afip_webservices import config
from .invoice import ElectronicInvoiceValidator


class WsfeInvoiceDetails(object):
    """
    Se encarga de asignar los detalles de una invoice en
    los factories de WSFE.

    :param client: Cliente / Webservice.
    :param invoice: Objeto ElectronicInvoice para tomar los detalles.
    :param last_invoice_number: Ultimo numero de comprobante.
    """

    def __init__(self, client, invoice, last_invoice_number):
        self.client = client
        self.invoice = invoice
        self.last_invoice_number = last_invoice_number
        self.detail = None

    def get_details(self):
        """ Devuelve los detalles completos para esa factura """

        self._set_details()

        return self.detail

    def _set_details(self):
        """ Completa los detalles a enviar segun los datos del documento """

        self.detail = self._get_detail()
        self.detail.Concepto = self.invoice.concept
        self.detail.DocTipo = self.invoice.customer_document_type
        self.detail.DocNro = self.invoice.customer_document_number
        self.detail.CbteDesde = self.last_invoice_number+1
        self.detail.CbteHasta = self.last_invoice_number+1
        self.detail.CbteFch = self.invoice.document_date
        self.detail.ImpTotal = round(self.invoice.get_total_amount(), 2)
        self.detail.ImpTotConc = round(self.invoice.untaxed_amount, 2)
        self.detail.ImpNeto = round(self.invoice.taxed_amount, 2)
        self.detail.ImpOpEx = round(self.invoice.exempt_amount, 2)
        self.detail.ImpIVA = round(self.invoice.get_total_iva(), 2)
        self.detail.ImpTrib = round(self.invoice.get_total_tributes(), 2)
        if self.invoice.concept not in [2, 3]:
            self.detail.FchServDesde = ''
            self.detail.FchServHasta = ''
            # Si es factura de crédito hay que informar siempre fecha de vencimiento
            self.detail.FchVtoPago = '' if self.invoice.document_code not in [201, 206, 211] else self.invoice.payment_due_date
        else:
            self.detail.FchServDesde = self.invoice.service_from
            self.detail.FchServHasta = self.invoice.service_to
            # Para notas de crédito / débito FCE no hay que informar fecha de vencimiento de pago.
            self.detail.FchVtoPago = self.invoice.payment_due_date if self.invoice.document_code not in [202, 203, 207, 208, 212, 213] \
                else None
        self.detail.MonId = self.invoice.mon_id
        self.detail.MonCotiz = round(self.invoice.mon_cotiz, 6)
        self._set_iva()
        self._set_tributes()
        self._set_optionals()
        self._set_associated_documents()

    def _serialize_tribute(self, tribute):
        return self._get_tribute()(
            Id=tribute.document_code,
            BaseImp=round(tribute.taxable_base, 2),
            Alic=tribute.aliquot,
            Importe=round(tribute.amount, 2),
            Desc='Impuesto codigo {}'.format(tribute.document_code),
        )

    def _set_tributes(self):
        """ Agrega al detalle el array de tributos del documento """

        if self.invoice.array_tributes:
            self.detail.Tributos = self._get_tribute_array()([
                self._serialize_tribute(tribute) for tribute in self.invoice.array_tributes
            ])

    def _serialize_iva(self, iva):
        return self._get_iva()(
            Id=iva.document_code,
            BaseImp=round(iva.taxable_base, 2),
            Importe=round(iva.amount, 2)
        )

    def _serialize_optional(self, optional):
        return self._get_optional()(
            Id=optional.optional_id,
            Valor=optional.value,
        )

    def _serialize_associated_document(self, associated_document):
        return self._get_associated_document()(
            Tipo=associated_document.document_type,
            PtoVta=associated_document.point_of_sale,
            Nro=associated_document.number,
            Cuit=associated_document.cuit,
            CbteFch=associated_document.document_date
        )

    def _set_iva(self):
        """ Agrega al detalle el array de iva del documento """

        if self.invoice.array_iva:
            self.detail.Iva = self._get_iva_array()([
                self._serialize_iva(iva) for iva in self.invoice.array_iva
            ])

    def _set_associated_documents(self):
        if self.invoice.associated_documents:
            self.detail.CbtesAsoc = self._get_associated_document_array()([
                self._serialize_associated_document(associated) for associated in self.invoice.associated_documents
            ])

    def _set_optionals(self):
        if self.invoice.array_optionals:
            self.detail.Opcionales = self._get_optional_array()([
                self._serialize_optional(optional) for optional in self.invoice.array_optionals
            ])

    def _get_detail(self):
        return self.client.type_factory('ns0').FECAEDetRequest()

    def _get_associated_document_array(self):
        return self.client.get_type('ns0:ArrayOfCbteAsoc')

    def _get_optional_array(self):
        return self.client.get_type('ns0:ArrayOfOpcional')

    def _get_associated_document(self):
        return self.client.get_type('ns0:CbteAsoc')

    def _get_optional(self):
        return self.client.get_type('ns0:Opcional')

    def _get_iva(self):
        return self.client.get_type('ns0:AlicIva')

    def _get_iva_array(self):
        return self.client.get_type('ns0:ArrayOfAlicIva')

    def _get_tribute(self):
        return self.client.get_type('ns0:Tributo')

    def _get_tribute_array(self):
        return self.client.get_type('ns0:ArrayOfTributo')


class WsfeOptional(object):

    def __init__(self, optional_id, value):
        self.optional_id = optional_id
        self.value = value


class WsfeAssociatedDocument(object):

    def __init__(self, document_type, point_of_sale, number, cuit, document_date):
        self.document_type = document_type
        self.point_of_sale = point_of_sale
        self.number = number
        self.cuit = cuit
        self._document_date = None
        self.document_date = document_date

    def _parse_date(self, value):
        return value.strftime('%Y%m%d')

    @property
    def document_date(self):
        return self._document_date

    @document_date.setter
    def document_date(self, value):
        self._document_date = self._parse_date(value)


class Wsfe(object):
    """
    Factura electronica.

    :param access_token: AccessToken - Token de acceso
    :param cuit: Cuit de la empresa
    :param homologation: Homologacion si es True
    :param url: Url de servicios para Wsfe
    """

    def __init__(self, access_token, cuit, homologation=True, url=None):
        if not url:
            self.url = config.service_urls.get('wsfev1_homologation') if homologation\
                else config.service_urls.get('wsfev1_production')
        else:
            self.url = url

        self.accessToken = access_token
        self.cuit = cuit
        self.auth_request = self._create_auth_request()

    def check_webservice_status(self):
        """ Consulta el estado de los webservices de AFIP."""

        res = Client(self.url).service.FEDummy()

        if hasattr(res, 'Errors'):
            raise AfipError.parse_error(res)
        if res.AppServer != 'OK':
            raise Exception('El servidor de aplicaciones no se encuentra disponible. Intente mas tarde.')
        if res.DbServer != 'OK':
            raise Exception('El servidor de base de datos no se encuentra disponible. Intente mas tarde.')
        if res.AuthServer != 'OK':
            raise Exception('El servidor de auntenticacion no se encuentra disponible. Intente mas tarde.')

    def get_cae(self, invoices, pos):
        """
        :param invoices: Conjunto de Objetos ElectronicInvoice, documentos a enviar a AFIP.
        :param pos: Numero de punto de venta.
        :returns str: Respuesta de AFIP sobre la validacion del documento.
        """

        self._validate_invoices(invoices)
        FECAERequest = self._set_FECAERequest(invoices, pos)
        # FECAESolicitar(Auth: ns0:FEAuthRequest, FeCAEReq: ns0:FECAERequest) ->
        # FECAESolicitarResult: ns0:FECAEResponse
        return Client(self.url).service.FECAESolicitar(
            Auth=self.auth_request,
            FeCAEReq=FECAERequest
        ), FECAERequest

    def get_cotization(self, currency):
        """
        :param currency: Id de la moneda consultar
        :return: Cotización de la moneda para el día de hoy.
        """
        cotiz_response = Client(self.url).service.FEParamGetCotizacion(
            Auth=self.auth_request,
            MonId=currency,
        )
        if cotiz_response.Errors:
            raise AfipError().parse_error(cotiz_response)

        return cotiz_response.ResultGet.MonCotiz

    def show_error(self, response):
        if response.Errors:
            raise AfipError.parse_error(response)

    def get_last_number(self, pos_number, document_type_number):
        """
        :param pos_number: Numero de punto de venta
        :param document_type_number: Numero del tipo de documento segun AFIP a consultar
        :return: Numero de ultimo comprobante autorizado para ese tipo
        """

        # FECompUltimoAutorizado(Auth: ns0:FEAuthRequest, PtoVta: xsd:int, CbteTipo: xsd:int) ->
        # FECompUltimoAutorizadoResult: ns0:FERecuperaLastCbteResponse

        last_number_response = Client(self.url).service.FECompUltimoAutorizado(
            Auth=self.auth_request,
            PtoVta=pos_number,
            CbteTipo=document_type_number,
        )
        if last_number_response.Errors:
            raise AfipError().parse_error(last_number_response)

        return last_number_response.CbteNro

    def _validate_invoices(self, invoices):
        """
        Valida que los campos de la factura electronica sean validos

        :param invoices: Lista de Objetos Invoice, documentos a validar.
        """

        invoiceValidator = ElectronicInvoiceValidator()
        for invoice in invoices:
            invoiceValidator.validate_invoice(invoice)

    def _get_header(self):
        return Client(self.url).get_type('ns0:FECAECabRequest')

    def _get_cae_request(self):
        return Client(self.url).get_type('ns0:FECAERequest')

    def _get_array_cae_request(self):
        return Client(self.url).get_type('ns0:ArrayOfFECAEDetRequest')

    def _get_document_type(self, invoices):
        document_types = set([invoice.document_code for invoice in invoices])
        if len(document_types) > 1:
            raise AttributeError("Los documentos a enviar deben ser del mismo tipo")

        return next(iter(document_types))

    def _set_FECAERequest(self, invoices, pos):
        """
        :param invoices: Conjunto de objetos ElectronicInvoice.
        :param pos: Numero de punto de venta.
        :returns: FECAERequest / Envio de documentos para recibir el CAE.
        """

        header = self._set_header(invoices, pos)
        array_cae_request = self._get_array_cae_request()

        details = []
        last_invoice = self.get_last_number(header.PtoVta, header.CbteTipo)

        for invoice in invoices:
            details.append(WsfeInvoiceDetails(Client(self.url), invoice, last_invoice).get_details())
            last_invoice += 1

        cae_request = self._get_cae_request()

        # ns0:FECAERequest(FeCabReq: ns0:FECAECabRequest, FeDetReq: ns0:ArrayOfFECAEDetRequest)
        FECAERequest = cae_request(
            FeCabReq=header,
            FeDetReq=array_cae_request(details)
        )

        return FECAERequest

    def _set_header(self, invoices, pos):
        """
        :param invoices: Conjunto de Objetos ElectronicInvoice.
        :param pos: Numero de punto de venta.
        :returns: FeCabReq / cabecera de envio de documentos completo.
        """

        header_request = self._get_header()
        # ns0:FECAECabRequest(CantReg: xsd:int, PtoVta: xsd:int, CbteTipo: xsd:int)
        header = header_request(
            CantReg=len(invoices),
            PtoVta=pos,
            CbteTipo=self._get_document_type(invoices)
        )

        return header

    def _create_auth_request(self):
        """ Setea el FEAuthRequest, necesario para utilizar el resto de los metodos """

        FEAuthRequest = Client(self.url).get_type('ns0:FEAuthRequest')
        # ns0:FEAuthRequest(Token: xsd:string, Sign: xsd:string, Cuit: xsd:long)
        auth_request = FEAuthRequest(
            Token=self.accessToken.token,
            Sign=self.accessToken.sign,
            Cuit=self.cuit
        )
        return auth_request



# -*- coding: utf-8 -*-
from .. import presentation
import unittest
import sys
sys.path.append("../")


class TestPurchaseSalesVatDigitalBookLine(unittest.TestCase):

    def _set_ventas_cbte_line(self, ventasCbte):

        ventasCbte.fecha = 20151030
        ventasCbte.tipo = '091'
        ventasCbte.puntoDeVenta = 0o0001
        ventasCbte.numeroComprobante = 4590
        ventasCbte.numeroHasta = 4600
        ventasCbte.codigoDocumento = 20
        ventasCbte.numeroComprador = 51231
        ventasCbte.denominacionComprador = 'Test Test'
        ventasCbte.importeTotal = 5800
        ventasCbte.importeTotalNG = 5000
        ventasCbte.percepcionNC = 300
        ventasCbte.importeExentos = 123
        ventasCbte.importePercepciones = 5123
        ventasCbte.importePerIIBB = 3331
        ventasCbte.importePerIM = 4234
        ventasCbte.importeImpInt = '12312'
        ventasCbte.codigoMoneda = '321'
        ventasCbte.tipoCambio = '4444000000'
        ventasCbte.cantidadAlicIva = 1
        ventasCbte.codigoOperacion = 1
        ventasCbte.otrosTributos = 0
        ventasCbte.fechaVtoPago = 20151130

    def _set_ventas_alicuotas_line(self, ventasAlicuotas):

        ventasAlicuotas.tipoComprobante = 91
        ventasAlicuotas.puntoDeVenta = 0o0001
        ventasAlicuotas.numeroComprobante = 9898989898
        ventasAlicuotas.importeNetoGravado = 10000
        ventasAlicuotas.alicuotaIva = 21
        ventasAlicuotas.impuestoLiquidado = 2100

    def _set_compras_cbte_line(self, comprasCbte):

        comprasCbte.fecha = 20161212
        comprasCbte.tipo = 98
        comprasCbte.puntoDeVenta = 0o001
        comprasCbte.numeroComprobante = '12345678'
        comprasCbte.despachoImportacion = 'despaimportacion'
        comprasCbte.codigoDocumento = 'CD'
        comprasCbte.numeroVendedor = '30709653543'
        comprasCbte.denominacionVendedor = 'Test test'
        comprasCbte.importeTotal = 100000
        comprasCbte.importeTotalNG = 0
        comprasCbte.importeOpExentas = 0
        comprasCbte.importePerOIva = 0
        comprasCbte.importePerOtrosImp = 0
        comprasCbte.importePerIIBB = 0
        comprasCbte.importePerIM = 0
        comprasCbte.importeImpInt = 1000
        comprasCbte.codigoMoneda = 'ARS'
        comprasCbte.tipoCambio = '1'
        comprasCbte.cantidadAlicIva = '5'
        comprasCbte.codigoOperacion = 'A'
        comprasCbte.credFiscComp = '0'
        comprasCbte.otrosTrib = '0'
        comprasCbte.cuitEmisor = '11022598'
        comprasCbte.denominacionEmisor = 'Responsable Inscripto'
        comprasCbte.ivaComision = '100'

    def _set_compras_alicuotas_line(self, comprasAlicuotas):

        comprasAlicuotas.tipoComprobante = '093'
        comprasAlicuotas.puntoDeVenta = '00005'
        comprasAlicuotas.numeroComprobante = 123516129872
        comprasAlicuotas.codigoDocVend = 13
        comprasAlicuotas.numeroIdVend = 26288940
        comprasAlicuotas.importeNetoGravado = 50000
        comprasAlicuotas.alicuotaIva = 3
        comprasAlicuotas.impuestoLiquidado = 10000

    def _set_compras_importaciones_line(self, comprasImportaciones):

        comprasImportaciones.despachoImportacion = '18273918273JKL'
        comprasImportaciones.importeNetoGravado = '100000'
        comprasImportaciones.alicuotaIva = '912'
        comprasImportaciones.impuestoLiquidado = '1000'

    def _set_credito_fiscal_importacion_serv_line(self, creditoFiscalImportacionServ):

        creditoFiscalImportacionServ.tipoComprobante = '3'
        creditoFiscalImportacionServ.descripcion = 'Descripcion'
        creditoFiscalImportacionServ.identificacionComprobante = '981273918CCLL'
        creditoFiscalImportacionServ.fechaOperacion = '20161212'
        creditoFiscalImportacionServ.montoMonedaOriginal = '10000'
        creditoFiscalImportacionServ.codigoMoneda = 'USD'
        creditoFiscalImportacionServ.tipoCambio = '15'
        creditoFiscalImportacionServ.cuitPrestador = '262841231'
        creditoFiscalImportacionServ.nifPrestador = '123123123245927'
        creditoFiscalImportacionServ.nombrePrestador = 'Nombre Apellido Prestador'
        creditoFiscalImportacionServ.alicuotaAplicable = '2100'
        creditoFiscalImportacionServ.fechaIngresoImpuesto = '20141213'
        creditoFiscalImportacionServ.montoImpuesto = '1300'
        creditoFiscalImportacionServ.impuestoComputable = '1000'
        creditoFiscalImportacionServ.idPago = '0001-8192839192'
        creditoFiscalImportacionServ.cuitEntidadPago = '30709612352'

    purchaseSalesVentasCbte = presentation.Presentation("libroIVADigital", "ventasCbte")
    purchaseSalesVentasAlicuotas = presentation.Presentation("libroIVADigital", "ventasAlicuotas")
    purchaseSalesComprasCbte = presentation.Presentation("libroIVADigital", "comprasCbte")
    purchaseSalesComprasAlicuotas = presentation.Presentation("libroIVADigital", "comprasAlicuotas")
    purchaseSalesComprasImportaciones = presentation.Presentation("libroIVADigital", "comprasImportaciones")
    purchaseSalesCreditoFiscalImportacionServ = presentation.Presentation("libroIVADigital",
                                                                          "creditoFiscalImportacionServ")


    def _create_ventas_cbte_lines(self):
        for i in range(3):
            self._set_ventas_cbte_line(self.purchaseSalesVentasCbte.create_line())

    def _create_ventas_alicuotas_lines(self):
        for i in range(5):
            self._set_ventas_alicuotas_line(self.purchaseSalesVentasAlicuotas.create_line())

    def _create_compras_cbte_lines(self):
        for i in range(1):
            self._set_compras_cbte_line(self.purchaseSalesComprasCbte.create_line())

    def _create_compras_alicuotas_lines(self):
        for i in range(2):
            self._set_compras_alicuotas_line(self.purchaseSalesComprasAlicuotas.create_line())

    def _create_compras_importaciones_lines(self):
        for i in range(7):
            self._set_compras_importaciones_line(self.purchaseSalesComprasImportaciones.create_line())

    def _create_credito_fiscal_importacion_serv_lines(self):
        for i in range(1):
            self._set_credito_fiscal_importacion_serv_line(self.purchaseSalesCreditoFiscalImportacionServ.create_line())

    def _create_lines(self):
        self._create_ventas_cbte_lines()
        self._create_ventas_alicuotas_lines()
        self._create_compras_cbte_lines()
        self._create_compras_alicuotas_lines()
        self._create_compras_importaciones_lines()
        self._create_credito_fiscal_importacion_serv_lines()

    def test_create_presentation_string(self):
        self._create_lines()

        # 3 de 266 y 3 de los \n y 3 de \r
        self.assertEqual(len(self.purchaseSalesVentasCbte.get_string()), 804)

        # 5 de 62 y 5 de los \n y 5 de \r
        self.assertEqual(len(self.purchaseSalesVentasAlicuotas.get_string()), 320)

        # 1 de 325 y 1\n y 1 de \r
        self.assertEqual(len(self.purchaseSalesComprasCbte.get_string()), 327)

        # 2 de 168 y 2 \n y 2 de \r
        self.assertEqual(len(self.purchaseSalesComprasAlicuotas.get_string()), 172)

        # 7 de 50 y 7 \n y 7 de \r
        self.assertEqual(len(self.purchaseSalesComprasImportaciones.get_string()), 364)

        # 1 de 211 y 1\n y 1 de \r
        self.assertEqual(len(self.purchaseSalesCreditoFiscalImportacionServ.get_string()), 213)


if __name__ == '__main__':
    unittest.main()
